/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#define MAX_CHART_ENTRY 50

#include "CollectionView/collectionscene.h"
#include "CollectionModel/collectionmodel.h"
#include "debug.h"


/*
********************************************************************************
*                                                                              *
*    Class CollectionScene                                                     *
*        -> dashboard method                                                   *
********************************************************************************
*/ 
void CollectionScene::populateDashBoardScene()
{
    Debug::debug() << " ---- CollectionScene::populateDashBoardScene";
    populateMostPlayedAlbum(0);
    populateTopRatedAlbum(itemsBoundingRect().height() + 40);
    populateMostPlayedArtist(itemsBoundingRect().height() + 40);
    populateTopRatedArtist(itemsBoundingRect().height() + 40);
}



/*******************************************************************************
    album part
*******************************************************************************/
void CollectionScene::populateMostPlayedAlbum(int YPos)
{
    int albumRow  = 0;
    int Column    = 0;
    int nbItem    = (_parentWidget->width()/160 > 2) ? _parentWidget->width()/160 : 2;
    int maxPlaycount = 0;
    
    int itemCount = 0;
   
    CategorieGraphicItem *cat_item = new CategorieGraphicItem();
    cat_item->_catName = QString(tr("Most played albums"));
    cat_item->_width   = _parentWidget->width() -20;
    cat_item->setPos(0 ,YPos);
    addItem(cat_item);
    
    //! get Album List from CollectionModel
    QList<MediaItemPtr> albumList = _collectionModel->albumItemList;

    //! sort 
    qSort(albumList.begin(), albumList.end(), MEDIA::compareAlbumItemPlaycount);
    
    //! loop over album MediaItem
    foreach(MediaItemPtr media_album, albumList) 
    {
      if(media_album->data.at(ALBUM::playcount).toInt() == 0 || itemCount == MAX_CHART_ENTRY) break;
      
      if(maxPlaycount == 0)
        maxPlaycount = media_album->data.at(ALBUM::playcount).toInt();
      
      if( !_collectionModel->isAlbumFiltered(media_album) ) continue;
      
      AlbumGraphicItem_v3 *album_item = new AlbumGraphicItem_v3();
      album_item->media = media_album;
      album_item->_maxPlaycount = maxPlaycount;      
      album_item->setPos(4+160*Column, YPos + 35 + albumRow*190);

      addItem(album_item);
      itemCount++;
      
      if(Column < (nbItem-1)) {
        Column++;
      } 
      else {
        Column = 0;
        albumRow++;
      }      
    }
    
    //! si liste vide --> message
    if(itemCount==0) {
      InfoGraphicItem *item = new InfoGraphicItem();
      item->_text = tr("No entry found");
      item->_width   = _parentWidget->width();
      item->setPos( 0 , YPos + 50);
      addItem(item);
    }
}

void CollectionScene::populateTopRatedAlbum(int YPos)
{
    int albumRow  = 0;
    int Column    = 0;
    int nbItem    = (_parentWidget->width()/160 > 2) ? _parentWidget->width()/160 : 2;
    int itemCount = 0;
    
    
    CategorieGraphicItem *cat_item = new CategorieGraphicItem();
    cat_item->_catName = QString(tr("Top rated albums"));
    cat_item->_width   = _parentWidget->width()-20;
    cat_item->setPos(0 ,YPos);
    addItem(cat_item);
    
    //! get Album List from CollectionModel
    QList<MediaItemPtr> albumList = _collectionModel->albumItemList;

    //! sort 
    qSort(albumList.begin(), albumList.end(), MEDIA::compareAlbumItemRating);
    
    //! loop over album MediaItem
    foreach(MediaItemPtr media_album, albumList) 
    {
      if(media_album->data.at(ALBUM::rating).toFloat() == 0 || itemCount == MAX_CHART_ENTRY) break;
      
      if( !_collectionModel->isAlbumFiltered(media_album) ) continue;
      
      AlbumGraphicItem_v4 *album_item = new AlbumGraphicItem_v4();
      album_item->media = media_album;
      album_item->setPos(4+160*Column, YPos + 35 + albumRow*190);

      addItem(album_item);
      itemCount++;
      
      if(Column < (nbItem-1)) {
        Column++;
      } 
      else {
        Column = 0;
        albumRow++;
      }      
    }    
    
    //! si liste vide --> message
    if(itemCount==0) {
      InfoGraphicItem *item = new InfoGraphicItem();
      item->_text = tr("No entry found");
      item->_width   = _parentWidget->width();
      item->setPos( 0 , YPos + 50);
      addItem(item);
    }
}
    


/*******************************************************************************
    artist part
*******************************************************************************/
void CollectionScene::populateMostPlayedArtist(int YPos)
{
    int artistRow  = 0;
    int Column     = 0;
    int nbItem = (_parentWidget->width()/160 > 2) ? _parentWidget->width()/160 : 2;
    QStringList coverPathList;
    int maxPlaycount = 0;
    int itemCount    = 0;
    
    
    CategorieGraphicItem *cat_item = new CategorieGraphicItem();
    cat_item->_catName = QString(tr("Most played artists"));
    cat_item->_width   = _parentWidget->width()-20;
    cat_item->setPos(0 ,YPos);
    addItem(cat_item);
    
    //! get artists List from CollectionModel
    QList<MediaItemPtr> artistList;
    for (int j=0 ; j < _collectionModel->rootItem()->childCount(); j++ ) {
       artistList << _collectionModel->rootItem()->child(j);
     }

    //! sort 
    qSort(artistList.begin(), artistList.end(), MEDIA::compareArtistItemPlaycount);

    
    //! loop over artist MediaItem
    foreach(MediaItemPtr media_artist, artistList) 
    {
      if(media_artist->data.at(ARTIST::playcount).toInt() == 0 || itemCount == MAX_CHART_ENTRY) break;
      
      if(maxPlaycount == 0)
        maxPlaycount = media_artist->data.at(ARTIST::playcount).toInt();
      
      if( !_collectionModel->isArtistFiltered(media_artist) ) continue;

      
      ArtistGraphicItem_v2 *artist_item = new ArtistGraphicItem_v2();
      artist_item->media = media_artist;
      artist_item->_maxPlaycount = maxPlaycount;
      artist_item->setPos(4+160*Column, YPos + 35 + artistRow*190);

      addItem(artist_item);
      itemCount++;
      
      //! loop over ALBUM (set cover path)
      for(int k = media_artist->childCount()-1 ; k >= 0; k--) {
        MediaItemPtr media_album = media_artist->child(k);
        coverPathList.prepend(media_album->data.at(ALBUM::coverpath).toString());
        if(coverPathList.size() >=6) break;
      }
      media_artist->data[ARTIST::coverpathlist] = coverPathList;
      coverPathList.clear();
      
      if(Column < (nbItem-1)) {
        Column++;
      } 
      else {
        Column = 0;
        artistRow++;
      }
    }
    
   
    //! si liste vide --> message
    if(itemCount==0) {
      InfoGraphicItem *info_item = new InfoGraphicItem();
      info_item->_text = tr("No entry found");
      info_item->_width   = _parentWidget->width();
      info_item->setPos( 0 , YPos + 50);
      addItem(info_item);
    }
}


void CollectionScene::populateTopRatedArtist(int YPos)
{
    int artistRow  = 0;
    int Column     = 0;
    int nbItem = (_parentWidget->width()/160 > 2) ? _parentWidget->width()/160 : 2;
    QStringList coverPathList;
    int itemCount = 0;
    
    
    CategorieGraphicItem *cat_item = new CategorieGraphicItem();
    cat_item->_catName = QString(tr("Top rated artists"));
    cat_item->_width   = _parentWidget->width()-20;
    cat_item->setPos(0 ,YPos);
    addItem(cat_item);
    
    //! get artists List from CollectionModel
    QList<MediaItemPtr> artistList;
    for (int j=0 ; j < _collectionModel->rootItem()->childCount(); j++ ) {
       artistList << _collectionModel->rootItem()->child(j);
     }

    //! sort 
    qSort(artistList.begin(), artistList.end(), MEDIA::compareArtistItemRating);

    //! loop over artist MediaItem
    foreach(MediaItemPtr media_artist, artistList) 
    {
      if(media_artist->data.at(ARTIST::rating).toFloat() == 0 || itemCount == MAX_CHART_ENTRY) break;
      
      if( !_collectionModel->isArtistFiltered(media_artist) ) continue;

      
      ArtistGraphicItem_v3 *artist_item = new ArtistGraphicItem_v3();
      artist_item->media = media_artist;
      artist_item->setPos(4+160*Column, YPos + 35 + artistRow*190);

      addItem(artist_item);
      itemCount++;
      
      //! loop over ALBUM (set cover path)
      for(int k = media_artist->childCount()-1 ; k >= 0; k--) {
        MediaItemPtr media_album = media_artist->child(k);
        coverPathList.prepend(media_album->data.at(ALBUM::coverpath).toString());
        if(coverPathList.size() >=6) break;
      }
      media_artist->data[ARTIST::coverpathlist] = coverPathList;
      coverPathList.clear();
      
      if(Column < (nbItem-1)) {
        Column++;
      } 
      else {
        Column = 0;
        artistRow++;
      }
    }    

    
    //! si liste vide --> message
    if(itemCount==0) {
      InfoGraphicItem *info_item = new InfoGraphicItem();
      info_item->_text = tr("No entry found");
      info_item->_width   = _parentWidget->width();
      info_item->setPos( 0 , YPos + 50);
      addItem(info_item);
    }
}

