/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _CONTEXT_WIDGETS_H_
#define _CONTEXT_WIDGETS_H_

#include "context_graphicitem.h"
#include "button_item.h"

#include "provider_discogs.h"
#include "provider_lyrics.h"

#include <QGraphicsWidget>
#include <QPixmap>
#include <QString>
#include <QByteArray>
#include <QGraphicsLinearLayout>



class LastFmProvider;
class EchonestProvider;

/*
********************************************************************************
*                                                                              *
*    Class ArtistInfoWidget                                                    *
*                                                                              *
********************************************************************************
*/ 
class ArtistInfoWidget : public QGraphicsWidget
{
Q_OBJECT
  public:
    ArtistInfoWidget(QWidget* parentView = 0,EchonestProvider* provider=0);
    
    void set_artist_name(const QString &);
    void set_request_id(int i) {m_request_id = i;}
    
    void clear();
    
    void update();
    
  private:
    void set_long_bio();
    void set_short_bio();
  
  private : 
    QWidget                 *m_parent;

    TitleGraphicItem        *m_title;
    QLabel                  *m_header;                   
    QLabel                  *m_image;
    WebLinkItem             *m_button;

    TextGraphicItem         *m_bio;    
    QString                  m_full_plain_bio;
    
    QGraphicsLinearLayout   *m_layout;
    QGraphicsLinearLayout   *m_layout_h;
    
    EchonestProvider        *m_provider;
    int                      m_request_id;    
    
  protected:
    void resizeEvent( QGraphicsSceneResizeEvent *event );
    QSizeF sizeHint(Qt::SizeHint which, const QSizeF &constraint = QSizeF()) const;    
    
  private slots:
    void slot_link_activated(const QString&);
    void slot_artist_bio_ready(int,QString, QString);
    void slot_artist_image_ready(int,QByteArray);
    
  signals:
    void updated();    
};

/*
********************************************************************************
*                                                                              *
*    Class ArtistSimilarWidget                                                 *
*                                                                              *
********************************************************************************
*/ 
class ArtistSimilarWidget : public QGraphicsWidget
{
Q_OBJECT
  public:
    ArtistSimilarWidget(QWidget* parentView = 0,EchonestProvider* provider=0);
    
    void set_artist_name(const QString &);
    void set_request_id(int i) {m_request_id = i;}
    void clear();
    void update();

  private :
    QSizeF doLayout(bool) const; 
      
  private : 
    QWidget                 *m_parent;
    QLabel                  *m_header;                   
    QList<TagGraphicItem*>   m_tags;
    
    EchonestProvider        *m_provider;
    int                      m_request_id;    
    
  protected:
    void resizeEvent( QGraphicsSceneResizeEvent *event );
    QSizeF sizeHint(Qt::SizeHint which, const QSizeF &constraint = QSizeF()) const;
        
  private slots:
    void slot_artist_similar_ready(int, QStringList);  
    
  signals:
    void updated();    
};

/*
********************************************************************************
*                                                                              *
*    Class AlbumInfoWidget                                                     *
*                                                                              *
********************************************************************************
*/ 
class AlbumInfoWidget : public QGraphicsWidget
{
Q_OBJECT

  public:
    AlbumInfoWidget(QWidget* parentView = 0,LastFmProvider* provider=0);
    
    void set_album_name(const QString &);
    void set_album_pix(const QPixmap &);
    void set_request_id(int i) {m_request_id = i;}
    
    void clear();
    void update();
    
  private : 
    QWidget                 *m_parent;
    
    TitleGraphicItem        *m_title;
    QLabel                  *m_image;
    QLabel                  *m_header;                   
    TextGraphicItem         *m_bio;
    
    QGraphicsLinearLayout   *m_layout;
    QGraphicsLinearLayout   *m_layout_h;
    
    LastFmProvider          *m_provider;
    int                      m_request_id;    
  
  protected:
    void resizeEvent( QGraphicsSceneResizeEvent *event );
    QSizeF sizeHint(Qt::SizeHint which, const QSizeF &constraint = QSizeF()) const;    
    
  private slots:
    void slot_album_bio_ready(int,QString);
    void slot_album_image_ready(int,QByteArray);

  signals:
    void updated();    
};

/*
********************************************************************************
*                                                                              *
*    Class DiscoInfoWidget                                                     *
*                                                                              *
********************************************************************************
*/
class DiscoInfoWidget : public QGraphicsWidget
{
Q_OBJECT
  public:
    DiscoInfoWidget(QWidget* parentView = 0,DiscogsProvider* provider=0);
    void set_request_id(int i) {m_request_id = i;}
    
    void clear();
    void update();
  
  private : 
    QSizeF doLayout(bool) const;
    
  private : 
    QWidget                 *m_parent;
    WebLinkItem             *m_button;    
    QLabel                  *m_header;
    
    /* map <disco release id, AlbumThumbGraphicItem*> */
    QMap<int, AlbumThumbGraphicItem*>   m_map_albums;
    /* data copy from discogs providers */
    DISCOGS::Artist          m_artist;
    
    DiscogsProvider         *m_provider;
    int                      m_request_id;    
  
  protected:
    void resizeEvent( QGraphicsSceneResizeEvent *event );
    QSizeF sizeHint(Qt::SizeHint which, const QSizeF &constraint = QSizeF()) const;
  
  private slots:
    void slot_artist_data_received(int,const DISCOGS::Artist&);
    void slot_cover_received(int,int release_id, QByteArray bytes);
                  
  signals:
    void updated();    
};

/*
********************************************************************************
*                                                                              *
*    Class LyricsInfoWidget                                                    *
*                                                                              *
********************************************************************************
*/ 
class LyricsInfoWidget : public QGraphicsWidget
{
Q_OBJECT

  public:
    LyricsInfoWidget(QWidget* parentView = 0,LyricsProvider* provider=0);
    
    void set_song_name(const QString &);
    void set_request_id(int i) {m_request_id = i;}
    
    void clear();
    void update();
    
  private : 
    QWidget                 *m_parent;
    
    TitleGraphicItem        *m_title;
    TextGraphicItem         *m_lyrics;
    QLabel                  *m_header;                   
    
    ButtonItem              *m_button_add;
    ButtonItem              *m_button_remove;
    WebLinkItem             *m_button_link;
        
    QGraphicsLinearLayout   *m_layout;
    QGraphicsLinearLayout   *m_layout_h;
    
    
    LyricsProvider          *m_provider;
    int                      m_request_id;    
    bool                     lyrics_found;
  
  protected:
    void resizeEvent( QGraphicsSceneResizeEvent *event );
    QSizeF sizeHint(Qt::SizeHint which, const QSizeF &constraint = QSizeF()) const;
    
  private slots:
    void slot_lyrics_ready(int id, LYRICS::Result& result);
    void slot_add_lyrics();
    void slot_remove_lyrics();
  
  signals:
    void updated();
};


/*
********************************************************************************
*                                                                              *
*    Class NowPlayingInfoWidget                                                *
*                                                                              *
********************************************************************************
*/ 
class NowPlayingInfoWidget : public QGraphicsWidget
{
Q_OBJECT
  public:
    NowPlayingInfoWidget(QWidget* parentView = 0);
  
  protected:
    void resizeEvent( QGraphicsSceneResizeEvent *event );
    QSizeF sizeHint(Qt::SizeHint which, const QSizeF &constraint = QSizeF()) const;

  public slots:
    void update();

  private : 
    QWidget                 *m_parent;
    
    TitleGraphicItem        *m_title;
    NowPlayingGraphicItem   *m_info;
    QGraphicsLinearLayout   *m_layout;
  
  signals:
    void updated();    
};


#endif // _CONTEXT_WIDGETS_H_
