/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "playlistdbwriter.h"
#include "playlistmodel.h"
#include "Database/database.h"
#include "MediaItem/mediaitem.h"
#include "debug.h"

//Qt
#include <QStringList>
#include <QtSql/QSqlDatabase>
#include <QtSql/QSqlQuery>
#include <QtSql/QSqlDriver>
#include <QtSql/QSqlField>
#include <QtSql/QSqlError>
#include <QtSql/QSqlResult>


/*
********************************************************************************
*                                                                              *
*    Class PlaylistDbWriter                                                    *
*                                                                              *
********************************************************************************
*/
PlaylistDbWriter::PlaylistDbWriter()
{
    _playlistModel = PlaylistModel::instance();
    _isRunning     = false;
    _playlist_name = QString();
    _database_id   = -1;
}


//! ----------------------- PlaylistDbWriter::saveToDatabase -------------------
void PlaylistDbWriter::saveToDatabase(const QString& playlist_name, int bd_id)
{
    _playlist_name = playlist_name;
    _database_id   = bd_id;
}

   
//! ----------------------- run ------------------------------------------------
void PlaylistDbWriter::run()
{
    _isRunning     = true;
    
    //! playlist data for Database
    QDateTime date   = QDateTime::currentDateTime();
    uint mtime       = date.toTime_t();
    QString pname    = "playlist-" + date.toString("dd-MM-yyyy-hh:mm:ss"); 
    QString fname    = QString(QCryptographicHash::hash(pname.toUtf8().constData(), QCryptographicHash::Sha1).toHex());
    int favorite     = 0;
    
    //! set real playlist name from user
    if(!_playlist_name.isEmpty()) {
      pname = _playlist_name;
    }
      
    
    QSqlField data("col",QVariant::String);

    Database db;
    if (!db.connect("PlaylistDbWriter")) return;
    
   
    if(_database_id != -1) 
    {
      QSqlQuery cleanQuery1("DELETE FROM `playlists` WHERE `id`="+QString::number(_database_id)+";", *db.sqlDb());
      QSqlQuery cleanQuery2("DELETE FROM `playlist_items` WHERE `playlist_id` NOT IN (SELECT `id` FROM `playlists`);", *db.sqlDb());
    }    
    
    
    Debug::debug() << "--- PlaylistDbWriter -> insert playlist" << pname;
    
    //! PLAYLIST part in database
    {
      QSqlQuery query(*db.sqlDb());
      query.prepare("INSERT INTO `playlists`(`filename`,`name`,`type`,`favorite`,`mtime`)" \
                    "VALUES(?," \
                    "       ?," \
                    "       ?," \
                    "       ?," \
                    "       ?);");
      query.addBindValue(fname);
      query.addBindValue(pname);
      query.addBindValue((int) T_DATABASE);
      query.addBindValue(favorite);
      query.addBindValue(mtime);
      query.exec();
    }
    
    data.setValue(fname);
    QString fnameDB = db.sqlDb()->driver()->formatValue(data,false);

    //! get Url from playlist 
    QStringList listUrl;
    for (int i = 0; i < _playlistModel->rowCount(QModelIndex()); i++) {
       listUrl << _playlistModel->MediaItemAt(i)->data.at(TRACK::url).toString(); // TRACK or STREAM
    }
    
    foreach (const QString &url, listUrl)
    {
      if(url.isEmpty()) continue;
      QString itemUrlName   = url; //QFileInfo(url).filePath().toUtf8();
      QString itemName      = QFileInfo(url).baseName();

      //! Playlist Item part in database
      {
        QSqlQuery itemQuery(*db.sqlDb());
        Debug::debug() << "--- PlaylistDbWriter -> insert playlistitem itemUrlName:" << itemUrlName;

        itemQuery.prepare("INSERT INTO `playlist_items`(`url`,`name`,`playlist_id`)" \
                          "VALUES(?," \
                          "       ?," \
                          "       (SELECT `id` FROM `playlists` WHERE `filename`="+fnameDB+"));");
        itemQuery.addBindValue(itemUrlName);
        itemQuery.addBindValue(itemName);
        itemQuery.exec();
      }  

    } // end foreach url into playlist
    
    _isRunning     = false;    
    _playlist_name = QString();
    
    emit playlistSaved();
}
