/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#include "Playlist/playlistpopulator.h"
#include "Playlist/playlistmodel.h"
#include "MediaItem/mediaitem.h"
#include "MediaItem/playlist_parser.h"
#include "networkaccess.h"
#include "utilities.h"
#include "debug.h"

//Qt
#include <QDataStream>
#include <QFile>
#include <QByteArray>
#include <QDirIterator>
#include <QMutexLocker>
#include <QFileInfo>


/*
********************************************************************************
*                                                                              *
*    Class PlaylistPopulator                                                   *
*                                                                              *
********************************************************************************
*/
PlaylistPopulator::PlaylistPopulator()
{
    m_model  = PlaylistModel::instance();
    _files.clear();
    m_medias.clear();
    _isRunning      = false;
    _playlist_row   = -1;
}

/*******************************************************************************
  PlaylistPopulator::run
*******************************************************************************/
void PlaylistPopulator::run()
{
  Debug::debug() << "PlaylistPopulator start";
  
  _isRunning = true;

  while (!_files.isEmpty() || !m_medias.isEmpty()) 
  {
      //Debug::debug() << "PlaylistPopulator process loop";

      /*--------------------------------------------------*/
      /* cas des dossiers                                 */
      /* -------------------------------------------------*/   
      if (_files.size() > 0) {      
        if (QFileInfo(_files.first()).isDir()) {
          const QString dirName = _files.takeFirst();
          const QStringList dirFilter  = QStringList() << "*.mp3" << "*.ogg" << "*.flac" << "*.wav" << "*.m4a" << "*.aac";
          QDirIterator dirIterator(dirName, dirFilter ,QDir::Files | QDir::NoSymLinks, QDirIterator::Subdirectories);

          QStringList readfiles; 
          while(dirIterator.hasNext()) {
            readfiles << dirIterator.next(); 
          }
        
          readfiles.append(_files);
          _files = readfiles;
        }
      }
     
      /*--------------------------------------------------*/
      /* cas des playlists remote                         */
      /* -------------------------------------------------*/
      if (_files.size() > 0) {
        if( !MEDIA::isUrlLocal(_files.first()) && 
            (MEDIA::isPlaylistFile(_files.first()) ||
             MEDIA::isShoutCastUrl(_files.first())) )
        {
            //! remote playlist --> download, make it local
            //Debug::debug() << "PlaylistPopulator startDownload" << _files.first();

            emit async_load(_files.takeFirst(), _playlist_row);
        }
      }

      /*--------------------------------------------------*/
      /* cas des playlists locale                         */
      /* -------------------------------------------------*/
      if (_files.size() > 0) {
      //! local playlist
        if( MEDIA::isUrlLocal(_files.first()) && MEDIA::isPlaylistFile(_files.first())) {
              //! local playlist
              Debug::debug() << "PlaylistPopulator PlaylistFromFile local " << _files.first();
              QList<MediaItemPtr> list = MEDIA::PlaylistFromFile(_files.takeFirst());

              foreach (MediaItemPtr mi, list) {
                if(mi->type == TYPE_TRACK) {
                    MediaItemPtr media = MEDIA::FromDataBase("PlaylistPopulatorConnection", mi->data.at(TRACK::url).toString());
                    if(!media) 
                      media = MEDIA::FromLocalFile(mi->data.at(TRACK::url).toString());
                    
                    m_model->insertMediaItem(media, _playlist_row);
                    media.reset();
                }
                else {
                    m_model->insertMediaItem(mi, _playlist_row);
                }
                mi.reset();
             } // foreach MediaItem
          }
      }

      /*--------------------------------------------------*/
      /* cas des fichiers et urls                         */
      /* -------------------------------------------------*/
      if (_files.size() > 0) {
          if(  MEDIA::isUrlLocal(_files.first()) && 
               MEDIA::isAudioFile(_files.first()) && 
              !MEDIA::isPlaylistFile(_files.first())) 
          {
              QString fileName = QFileInfo(_files.takeFirst()).canonicalFilePath();
      
              MediaItemPtr media = MEDIA::FromDataBase("PlaylistPopulatorConnection", fileName);
              if(!media) 
                media = MEDIA::FromLocalFile(fileName);

              m_model->insertMediaItem(media, _playlist_row);
              media.reset();
          }
          else if(!MEDIA::isUrlLocal(_files.first())) {
              //! remote file
              Debug::debug() << "remote uRL" << _files.first();
              QString url = _files.takeFirst();
              MediaItemPtr mi = MediaItemPtr(new MediaItem(TYPE_STREAM));
              mi->data[STREAM::id]          = -1;
              mi->data[STREAM::url]         = url;
              mi->data[STREAM::name]        = url;
              mi->data[STREAM::title]       = QString();
              mi->data[STREAM::artist]      = QString();
              mi->data[STREAM::album]       = QString();
              mi->data[STREAM::categorie]   = QString();
              mi->data[STREAM::isFavorite]  = false;
              mi->data[STREAM::isPlaying]   = false;
              mi->data[STREAM::isBroken]    = false;
              mi->data[STREAM::isPlayed]    = false;
              mi->data[STREAM::isStopAfter] = false;
                   
             m_model->insertMediaItem(mi, _playlist_row);
             mi.reset();
          }
          else {
              //!WARNING on doit sortir les éléments non traité (si boucle while)
              Debug::debug() << "PlaylistPopulator --> unsupported media !" << _files.takeFirst();
          }
      }
      
      /*--------------------------------------------------*/
      /* cas des mediaitems                               */
      /* -------------------------------------------------*/
      if(m_medias.size() > 0) {
        //Debug::debug() << "PlaylistPopulator m_medias.size() :" << m_medias.size();
        
        MediaItemPtr media = m_medias.takeFirst();
        if( MEDIA::isPlaylistFile(media->data.at(STREAM::url).toString()) ||
            MEDIA::isShoutCastUrl(media->data.at(STREAM::url).toString()) )
          {
            _files.append( media->data.at(STREAM::url).toString() );
          }
          else 
          {
            m_model->insertMediaItem(media, _playlist_row); 
          }
      }
      
  } //! END !_files.isEmpty() && !m_medias.isEmpty()
  
  m_model->removeDuplicate();

  _isRunning = false;
  emit playlistPopulated();
  Debug::debug() << "PlaylistPopulator END";  
}

/*******************************************************************************
  User methode
*******************************************************************************/   
/* 
  addFile  : add one file to proceed
  addFiles : add files
  addUrls  : add Urls
*/

void PlaylistPopulator::addFile(const QString &file)
{
    //Debug::debug() << "PlaylistPopulator append file :" << file;
    _playlist_row = -1;
    _files.append(file);
}

void PlaylistPopulator::addFiles(const QStringList &files)
{
    //Debug::debug() << "PlaylistPopulator append files :" << files;
    _playlist_row = -1;
    _files.append(files);
}

void PlaylistPopulator::addUrls(QList<QUrl> listUrl, int playlist_row)
{
    //Debug::debug() << "PlaylistPopulator append url :" << listUrl;
    _playlist_row = playlist_row;
    
    foreach (const QUrl &url, listUrl) {
      if(!url.toString().contains(QRegExp("^http://")))
        _files.append(url.toLocalFile());
      else
        _files.append(url.toString());
    }
}

void PlaylistPopulator::addMediaItems(QList<MediaItemPtr> list, int playlist_row)
{
    //Debug::debug() << "PlaylistPopulator addMediaItems " << list;
    _playlist_row = playlist_row;
    m_medias.append(list);
}
    
/*
********************************************************************************
*                                                                              *
*    Class AsynchronousLoadTask                                                *
*                                                                              *
********************************************************************************
*/
AsynchronousLoadTask::AsynchronousLoadTask(const QString& url, int row)
{  
    m_url  = url;
    m_row  = row;
}

void AsynchronousLoadTask::start_asynchronous_download()
{
    Debug::debug() << "AsynchronousLoadTask start_asyncronous_download";

    QObject *reply = HTTP()->get(QUrl(m_url));
    connect(reply, SIGNAL(error(QNetworkReply*)), this, SLOT(slot_download_error()));
    connect(reply, SIGNAL(data(QByteArray)), SLOT(slot_download_done(QByteArray)));     
}


void AsynchronousLoadTask::slot_download_done(QByteArray bytes)
{
    Debug::debug() << "AsynchronousLoadTask slot_download_done";

    if(bytes.isEmpty()) 
    {
      Debug::debug() << "AsynchronousLoadTask received bytes";
    }
    else 
    {
      QList<MediaItemPtr> list = MEDIA::PlaylistFromBytes(bytes);
      
      foreach (MediaItemPtr mi, list) 
      {
        if(mi->type == TYPE_TRACK) {
          MediaItemPtr media = MEDIA::FromDataBase("AsynchronousLoadTask", mi->data.at(TRACK::url).toString());
          if(!media) 
            media = MEDIA::FromLocalFile(mi->data.at(TRACK::url).toString());
                    
          m_model->insertMediaItem(media, m_row);
          media.reset();
        }
        else {
          m_model->insertMediaItem(mi, m_row);
        }
      mi.reset();
      } // foreach MediaItem            
    }
    
    delete this;
}

void AsynchronousLoadTask::slot_download_error()
{
    Debug::debug() << "AsynchronousLoadTask slot_download_error";
    MediaItemPtr media = MediaItemPtr(new MediaItem(TYPE_STREAM));
    media->data[STREAM::id]          = -1;
    media->data[STREAM::url]         = m_url;
    media->data[STREAM::name]        = m_url;
    media->data[STREAM::isFavorite]  = false;
    media->data[STREAM::isPlaying]   = false;
    media->data[STREAM::isBroken]    = true;
    media->data[STREAM::isPlayed]    = false;
    media->data[STREAM::isStopAfter] = false;
 
    m_model->insertMediaItem(media, m_row);
    
    delete this;
}

