/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _PLAYLIST_VIEW_H_
#define _PLAYLIST_VIEW_H_

#include "Playlist/playlistmodel.h"
#include "MediaItem/mediaitem.h"


#include <QtGui>
#include <QString>

/*******************************************************************************
  Playqueue mode (manage by PlaylistWidget)
*******************************************************************************/
namespace PLAYQUEUE {
  enum Q_MODE {
      MODE_TITLE      = 1,
      MODE_ALBUM      = 2,
      MODE_ARTIST     = 3,
      MODE_EXTENDED   = 4
  };
}

/*
********************************************************************************
*                                                                              *
*    Class PlaylistDelegate                                                    *
*                                                                              *
********************************************************************************
*/
// delegate to have custom painting for track into playlist widget :
//   - paint specific icon for playing item
//   - paint specific icon for broken url item
//   - paint information (artist-album-title) depending on mode
//   - paint duration information
//   - paint simple remote url (with network platform icon)
class PlaylistDelegate : public QItemDelegate
{
  public:
    PlaylistDelegate(QObject *parent);
  
    void paint ( QPainter * painter, const QStyleOptionViewItem & option, const QModelIndex & index ) const;
    QSize sizeHint(const QStyleOptionViewItem &option,const QModelIndex &index) const;

    //! get & set mode
    int mode() {return (int) _mode;}
    void setMode(int m) {_mode = (PLAYQUEUE::Q_MODE)m;}
    
  private :
    QString   getTrackInfo(const MediaItemPtr mi) const;
    QIcon     getIcon(const MediaItemPtr mi) const;
    void      drawStop(QPainter * painter, QRect rect) const;
    
    QIcon     icon_media_playing;
    QIcon     icon_media_broken;
    QIcon     icon_media_track;
    QIcon     icon_media_stream;
    
    QFont     font_normal;
    QFont     font_bold;    
    
    PLAYQUEUE::Q_MODE  _mode;
};


/*
********************************************************************************
*                                                                              *
*    Class PlaylistView                                                        *
*                                                                              *
********************************************************************************
*/
class PlaylistModel;

class PlaylistView : public QListView
{
Q_OBJECT
  public: 
    PlaylistView(QWidget *parent, PlaylistModel* model);
    
    void clearPlaylist();
    const MediaItemPtr getActiveMediaItem();
    const MediaItemPtr getNextMediaItem();
    const MediaItemPtr getPrevMediaItem();
    const MediaItemPtr getMediaItemAt(int);
    const MediaItemPtr getSelectedMediaItem();
    
    bool isMediaItemSelected() {return !_selection.isEmpty();}
    void stopPlaying();

    void keyPressEvent(QKeyEvent* event);
    
    //! get & set mode
    int mode() {return _playlistDelegate->mode();} 
    void setMode(int m) {_playlistDelegate->setMode(m);} 
    
  protected:
    void paintEvent(QPaintEvent *event);
    void dragMoveEvent(QDragMoveEvent *event);
    void dragEnterEvent(QDragEnterEvent *event);
    void dragLeaveEvent(QDragLeaveEvent *event);
    void dropEvent(QDropEvent *event);
    void mouseMoveEvent(QMouseEvent* event);
    
  private :
    enum RepeatMode {RepeatOff = 0,RepeatTrack = 1, RepeatAll = 2 };
    enum ShuffleMode {ShuffleOff = 0 ,ShuffleOn = 1};

    PlaylistModel     *_playlistModel;
    PlaylistDelegate  *_playlistDelegate;
    
    QModelIndexList    _selection;
    int                drop_indicator_row_;
    bool               drag_over_;

  public slots:
    void selectionChanged (const QItemSelection & selected, const QItemSelection & deselected);
    
  private slots:
    void slot_itemActivated(const QModelIndex &index);
    void removeSelected();
    void selectItems(QList<MediaItemPtr>);
    void sendLastFmLove();
    
  signals:
    void selectionChanged();
    void signal_playlist_itemDoubleClicked();
    void signal_lastFmLove(QString, QString);
};


#endif // _PLAYLIST_VIEW_H_
