/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


//! local
#include "Scrobbler/scrobblermanager.h"

#include "MediaItem/mediaitem.h"
#include "yarocksettings.h"  
#include "utilities.h"  
#include "global_actions.h" 
#include "debug.h" 


//! Qt
#include <QSettings>
#include <QString>
#include <QAction>
#include <QDateTime>


/*
********************************************************************************
*                                                                              *
*    Class ScrobblerManager                                                    *
*                                                                              *
********************************************************************************
*/
ScrobblerManager::ScrobblerManager(QObject *parent)  : QObject(parent)
{
    //Debug::debug() << "--- ScrobblerManager ->creation";
    _player         = EnginePlayer::instance();
    _scrobbler      = 0;    
    _engineOldState = ENGINE::STOPPED;
    
    settingsChanged();  
}

ScrobblerManager::~ScrobblerManager()
{
    //Debug::debug() << "--- ScrobblerManager ->delete";
    if(_scrobbler)
      delete _scrobbler;
}



void ScrobblerManager::settingsChanged()
{
    bool enable = SETTINGS()->_useLastFmScrobbler;

    if(!enable) 
    {
      disconnect(_player, 0,this, 0);

      if(_scrobbler) {
        delete _scrobbler;
        _scrobbler = 0;            
      }
    }    
    else if(enable) 
    {
      //! start scrobbling
      Debug::debug() << "--- ScrobblerManager -> start lastfm scrobbler";
      if(_scrobbler == 0) {
        _scrobbler     = new LastFm::Scrobbler();
      }
      
      if( _scrobbler->isAuthenticated() ) {
        connect(_player, SIGNAL(engineStateChanged()), this, SLOT(slotStateChanged()));
        connect(_player, SIGNAL(playingInfoChanged()), this, SLOT(trackChanged()));
      } 
    }
    
    //! change status for send love action
    ACTIONS()->value(LASTFM_LOVE)->setEnabled(enable);
    ACTIONS()->value(LASTFM_LOVE_NOW_PLAYING)->setEnabled(enable);    
}


void ScrobblerManager::trackChanged()
{
    MediaItemPtr mi = _player->enginePlayingMediaItem();

    //! Submit the old track
    if (_scrobbler->currentTrack())
        _scrobbler->currentTrack()->scrobble();

    if(mi->type != TYPE_TRACK) return;

    uint stamp = QDateTime::currentDateTime().toTime_t();

    LastFm::Track *track = new LastFm::Track(_scrobbler);
    
    track->setArtist( mi->data.at(TRACK::artist).toString() );
    track->setTrackTitle( mi->data.at(TRACK::title).toString() );
    track->setAlbum( mi->data.at(TRACK::album).toString() );
    track->setTrackLength(mi->data.at(TRACK::duration).toInt());
    track->setGenre( mi->data.at(TRACK::genre).toString() );
    track->setTrackNumber( int( mi->data.at(TRACK::num).toInt() ) );
    track->setPlaybackStart(stamp);
    
    //! set current track
    _scrobbler->setCurrentTrack(track);

    //! Set "Now playing"
    track->nowPlaying();
}

void ScrobblerManager::slotStateChanged()
{
    if ( _engineOldState == ENGINE::PAUSED && _player->engineState() == ENGINE::PLAYING)
        if (_scrobbler->currentTrack())
            _scrobbler->currentTrack()->pause(false);

    if ( _engineOldState == ENGINE::PLAYING && _player->engineState() == ENGINE::PAUSED)
        if (_scrobbler->currentTrack())
            _scrobbler->currentTrack()->pause(true);
            
    _engineOldState = _player->engineState();
}



void ScrobblerManager::loveTrackRequest(QString artist, QString title)
{
    Debug::debug() << "--- ScrobblerManager :loveTrackRequest track :" << title;
    LastFm::Track *track = new LastFm::Track(_scrobbler);
    track->setTrackTitle(title);
    track->setArtist(artist);
    
    track->love();
    connect(track, SIGNAL(loved()),track, SLOT(deleteLater()));
}
