/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef _SETTINGS_SCENE_H_
#define _SETTINGS_SCENE_H_


#include <QGraphicsScene>
#include <QGraphicsLinearLayout>
#include <QMap>

namespace SETTINGS {

enum SETTINGS_PAGES {
      GENERAL,
      PLAYER,
      LIBRARY,
      RADIO,
      SHORTCUT,
      SCROBBLER
      };

struct Results {
      bool    isSystrayChanged;
      bool    isDbusChanged;
      bool    isMprisChanged;

      bool    isCollectionChanged;
      bool    isRadioChanged;
      bool    isShorcutChanged;
      bool    isScrobblerChanged;
     };

};


class ButtonItem;
class ButtonStateItem;
class SettingsMessage;
/*
********************************************************************************
*                                                                              *
*    Class SettingsScene                                                       *
*                                                                              *
********************************************************************************
*/ 
class SettingsScene : public QGraphicsScene 
{
Q_OBJECT
  public:
    SettingsScene(QWidget* parent);
    ~SettingsScene();
    
    void init();
    void populate();
    void restore_settings();
    
    bool isInitialized() {return m_initialized;}

    const SETTINGS::Results results() {return m_result;}
    
    void set_page(SETTINGS::SETTINGS_PAGES);
    
  private slots:
    void slot_change_page();
    void slot_apply_settings();
    void slot_cancel_settings();
    
  private :
    QWidget                      *m_parentWidget;
    bool                          m_initialized;
    
    QGraphicsLinearLayout        *m_layout;
    SettingsMessage              *m_message;

    SETTINGS::SETTINGS_PAGES      m_currentPage;
    SETTINGS::Results             m_result;
    
    ButtonItem                   *m_apply_button;
    ButtonItem                   *m_cancel_button;
    
    QMap<SETTINGS::SETTINGS_PAGES, ButtonStateItem*>  m_buttons;
    QMap<SETTINGS::SETTINGS_PAGES, QGraphicsWidget*>  m_pages;    
  
  signals:
    void settings_saved();
};


#endif // _SETTINGS_SCENE_H_
