/* YaRock Player
* Copyright (C) 2010-2013 Sebastien amardeilh (sebastien.amardeilh+yarock@gmail.com)
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _MAINWINDOW_H_
#define _MAINWINDOW_H_

#include "threadmanager.h"

// qt
#include <QMainWindow>
#include <QSystemTrayIcon>



class CommandlineOptions;

// player engine
class EnginePlayer;

// widgets
class PlaylistView;
class PlaylistWidget;
class NowPlayingView;
class GraphicViewWidget;
class CentralWidget;
class MenuView;
class MenuModel;
class MinimalWidget;
class StatusWidget;
class CentralToolBar;

// Model
class CollectionModel;
class PlsFileModel;
class StreamModel;
class HistoModel;
class PlaylistModel;

// Views
class CollectionScene;
class StreamScene;
class SettingsScene;
// Core 
class ScrobblerManager;
class ThreadManager;
class HistoManager;
class DatabaseManager;
class VirtualPlayqueue;
class GlobalShortcuts;

// Dbus & Mpris
class DbusNotification;
class Mpris1;

/*
********************************************************************************
*                                                                              *
*    Class MainWindow                                                          *
*                                                                              *
********************************************************************************
*/
class MainWindow : public QMainWindow 
{
Q_OBJECT
    static MainWindow         *INSTANCE;
    
  public:
    MainWindow(QWidget *parent = 0);
    ~MainWindow();
    static MainWindow* instance() { return INSTANCE; }
    
    void commandlineOptionsHandle(const CommandlineOptions& options);

  public slots: 
    /* public slots for Mpris access */
    void playOrPause();
    void stopPlayer();
    void playNext();
    void playPrev();
    
  private:
    /* Mainwindow        */
    void createGui();
    void createActions();
    void connectSlots();

    /* Settings          */
    void reloadMprisSettings();
    void reloadSystraySettings();    
    void savePlayingTrack();
    void restorePlayingTrack();
            
    /* Database Method   */
    void removeDatabase();
    void createDatabase();
    void rebuildDatabase();
                
  private slots:
    /* Mainwindow        */  
    void slot_systray_clicked(QSystemTrayIcon::ActivationReason);
    void slot_widget_mode_switch();
    void slot_commandline_received(const QByteArray& serialized_options);
    void slot_on_settings_saved();
    void slot_on_aboutYarock();
    void slot_on_aboutQt();
    void slot_on_yarock_quit();
          
    /* Playqueue        */
    void slot_playqueue_add();
    void slot_playqueue_clear();
    void slot_playqueue_save();
    void slot_playqueue_save_auto();
        
    /*  Player           */
    void slot_player_enqueue_next();
    void slot_player_on_state_change();
    void slot_play_from_playqueue();
    void slot_play_from_collection();
    void slot_player_on_track_change (const QString& file);
    void slot_stop_after_media_triggered();
            
    /* Database slots    */
    void slot_database_start();
    void slot_database_dialog();

    /* Data models & views  */
    void slot_model_populate();
    void slot_model_populate(int model);
    void slot_views_update(E_MODEL_TYPE model_updated);
            
    /* Last FM           */      
    void slot_LastFmLove();
    
    /* Playlist editor   */
    void slot_open_playlist_editor();
    void slot_delete_playlist_editor();
    
    /* Jump to item slot */
    void slot_jump_to_now_playing();
            
    /* Equalizer         */
    void slot_eq_openDialog();
    void slot_eq_enableChange(bool eqActivated);
    void slot_eq_paramChange(int, QList<int>);

    /*  cover thread     */
    void startCoverSearchThread();
  
  protected:
    void resizeEvent(QResizeEvent *event);
    void closeEvent(QCloseEvent *event);

  private:
    enum E_PLAYING_FROM   {FromCollection, FromPlayQueue};
    
    E_PLAYING_FROM        _playRequestFrom;

    EnginePlayer          *_player;
    
    StatusWidget          *_statusWidget;

    CentralToolBar        *_centralToolBar;
    CentralWidget         *_centralWidget;
    MenuView              *_menuView;
    MenuModel             *_menuModel;

    PlaylistView          *_playlistView;
    PlaylistModel         *_playlistModel;
    PlaylistWidget        *_playlistwidget;
    NowPlayingView        *_nowplayingview;
  
    CollectionModel       *_collectionModel;
    PlsFileModel          *_plsFileModel;
    HistoModel            *_histoModel;
    CollectionScene       *_collectionScene;
    GraphicViewWidget     *_graphicViewWidget;

    VirtualPlayqueue      *_virtualPlayqueue;
      
    StreamModel           *_streamModel;
    StreamScene           *_streamScene;
    SettingsScene         *m_settingsScene;

    ThreadManager         *_threadManager;
    
    DatabaseManager       *_dbManager;
  
    QIcon                  *_appIcon;
    QSystemTrayIcon        *_trayIcon;
    bool                   _canClose;
    
    MinimalWidget          *_minimalwidget;    
    HistoManager           *_histoManager;
    
    ScrobblerManager       *_scrobblerManager;
    
    GlobalShortcuts        *_globalShortcuts;
    DbusNotification       *_dbusNotification;
    Mpris1                 *_mpris1;

signals:
    void playlistFinished();
};

#endif // _MAINWINDOW_H_
